==> xl-react-components-0.18.103.tgz: package/app/xl-react-widget/xl-react-widget-tree-select/xl-react-widget-tree-select.js <==
import R from 'ramda';
import React, {Component} from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';
import VirtualizedSelect from 'react-virtualized-select';
import XlReactWidgetTreeSelectItem from './xl-react-widget-tree-select-item';
import {
    changeScrollPosition,
    filterTree,
    flatTree,
    initTree,
    setSelectedNode,
    SCROLL_TO_BOTTOM,
    SCROLL_TO_TOP,
    toggleNode
} from './tree-utils';

import './xl-react-widget-tree-select.less';
import 'react-select/dist/react-select.css';

export class XlReactWidgetTreeSelect extends Component {
    static propTypes = {
        clearable: PropTypes.bool,
        data: PropTypes.arrayOf(PropTypes.shape({
            id: PropTypes.string.isRequired,
            title: PropTypes.string.isRequired
        })),
        onModelChange: PropTypes.func,
        selectedFolderId: PropTypes.string
    };

    static defaultProps = {clearable: true, data: []};

    static getDerivedStateFromProps(props, state) {
        const {data, selectedFolderId} = props;
        if (state.selectedFolderId === selectedFolderId) {
            return null;
        }
        return {
            ...state,
            selectedFolderId: selectedFolderId,
            tree: setSelectedNode(initTree(data), selectedFolderId)
        };
    }

    constructor(props) {
        super(props);
        this.dropdownRef = React.createRef();
        this.focusedItemRef = React.createRef();
        this.state = this.initialState();
    }

    componentDidUpdate() {
        this.scroll2item();
    }

    /* eslint-disable complexity */
    handleItemSelect = (folderSelected) => {
        const {selectedFolderId} = this.props;
        const {tree} = this.state;

        if (!R.pathEq(['id'], selectedFolderId, folderSelected)) {
            const selectedId = R.prop('id', folderSelected);
            const selectedTitle = R.prop('title', folderSelected);
            const isSelectable = R.prop('enabled', folderSelected);

            this.setState({tree: setSelectedNode(tree, selectedFolderId)});

            if (this.props.onModelChange && isSelectable) {
                this.props.onModelChange({selectedId, title: selectedTitle});
            } else if (R.isNil(selectedId)) {
                this.props.onModelChange({selectedId, title: ''});
            }
        }
    };
    /* eslint-enable */

    handleBlur = () => {
        const {selectedFolderId} = this.props;
        if (selectedFolderId) {
            const {tree} = this.state;
            this.setState({tree: setSelectedNode(tree, selectedFolderId)});
        }
    };

    handleItemToggle = (id, event) => {
        event.stopPropagation();
        const {tree} = this.state;
        this.setState({tree: toggleNode(tree, id)});
    };

    handleInputChange = (inputValue) => {
        const {data, selectedFolderId} = this.props;

        if (inputValue.trim()) {
            this.setState({tree: filterTree(data, inputValue)});
        } else if (selectedFolderId) {
            this.setState({tree: setSelectedNode(initTree(data), selectedFolderId)});
        } else {
            this.setState({tree: initTree(data)});
        }

        return inputValue;
    };

    /* eslint-disable complexity, default-case, max-depth */
    handleKeyDown = (event) => {
        const {selectedFolderId} = this.props;
        const {tree} = this.state;

        if (!this.dropdownRef.current._selectRef || !this.dropdownRef.current._selectRef._focusedOption) {
            return;
        }
        const {_focusedOption: {id: focusedId}} = this.dropdownRef.current._selectRef;

        const focused = focusedId || selectedFolderId || 0;
        const {keyCode} = event;
        const folders = flatTree(tree);

        const idx = folders.findIndex((folder) => folder.id === focused);

        if (idx >= 0) {
            switch (keyCode) {
                // [enter]
                case 13:
                    if (this.dropdownRef.current._selectRef.state.isOpen) {
                        event.preventDefault();
                        this.dropdownRef.current._selectRef.selectValue(folders[idx]);
                    }
                    break;
                // ←
                case 37: {
                    event.preventDefault();
                    const collapsed = toggleNode(tree, focused, false);
                    this.setState({tree: collapsed});
                    break;
                }
                // ↑
                case 38: {
                    event.preventDefault();
                    const nextItemIdx = idx === 0 ? folders.length - 1 : idx - 1;
                    this.dropdownRef.current._selectRef.focusOption(folders[nextItemIdx]);
                    this.setState({
                        needScrollTo: idx < nextItemIdx && nextItemIdx === folders.length - 1 ?
                            SCROLL_TO_BOTTOM :
                            undefined
                    });
                    break;
                }
                // →
                case 39: {
                    event.preventDefault();
                    const expanded = toggleNode(tree, focused, true);
                    this.setState({tree: expanded});
                    break;
                }
                // ↓
                case 40: {
                    event.preventDefault();
                    const nextItemIdx = idx === folders.length - 1 ? 0 : idx + 1;
                    this.dropdownRef.current._selectRef.focusOption(folders[nextItemIdx]);
                    this.setState({
                        needScrollTo: idx > nextItemIdx && nextItemIdx === 0 ?
                            SCROLL_TO_TOP :
                            undefined
                    });
                    break;
                }
            }
        }
    };
    /* eslint-enable */

    handleSelect = (selectValue, option) => () => {
        selectValue(option);
    };

    handleFocus = (focusOption, option) => () => {
        focusOption(option);
    };

    assignFocusedItem = (option, focusedOption) => (ref) => {
        if (option.id === focusedOption.id) {
            this.focusedItemRef = ref;
        }
    };

    /* eslint-disable react/no-find-dom-node */
    scroll2item = () => {
        const {needScrollTo} = this.state;
        const container = ReactDOM
            .findDOMNode(this.dropdownRef.current)
            .getElementsByClassName('VirtualSelectGrid')[0];
        changeScrollPosition(this.focusedItemRef, container, needScrollTo);
    };
    /* eslint-enable */

    initialState = () => {
        const {data, selectedFolderId} = this.props;
        const tree = initTree(data);
        return {
            selectedFolderId,
            tree: selectedFolderId ? setSelectedNode(tree, selectedFolderId) : tree
        };
    };

    optionRenderer = (params) => {
        const {
            key, option, focusedOption, focusOption, selectValue, style
        } = params;

        return <XlReactWidgetTreeSelectItem
            isFocused={option.id === focusedOption.id}
            key={key}
            node={option}
            onMouseEnter={this.handleFocus(focusOption, option)}
            onSelect={this.handleSelect(selectValue, option)}
            onToggle={this.handleItemToggle}
            ref={this.assignFocusedItem(option, focusedOption)}
            style={style}
        />;
    };

    render() {
        const {selectedFolderId} = this.props;
        const {tree} = this.state;

        return <VirtualizedSelect
            clearable={this.props.clearable}
            filterOptions={(options) => options}
            labelKey="title"
            onBlur={this.handleBlur}
            onBlurResetsInput={false}
            onChange={this.handleItemSelect}
            onInputChange={this.handleInputChange}
            onInputKeyDown={this.handleKeyDown}
            optionHeight={29}
            optionRenderer={this.optionRenderer}
            options={flatTree(tree)}
            ref={this.dropdownRef}
            simpleValue={false}
            value={selectedFolderId}
            valueKey="id"
        />;
    }
}

==> xl-react-components-0.21.9.tgz: package/app/xl-react-widget/xl-react-widget-tree-select/xl-react-widget-tree-select.js <==
import R from 'ramda';
import React, {Component} from 'react';
import ReactDOM from 'react-dom';
import PropTypes from 'prop-types';
import VirtualizedSelect from 'react-virtualized-select';
import XlReactWidgetTreeSelectItem from './xl-react-widget-tree-select-item';
import {
    changeScrollPosition,
    filterTree,
    flatTree,
    initTree,
    setSelectedNode,
    SCROLL_TO_BOTTOM,
    SCROLL_TO_TOP,
    toggleNode
} from './tree-utils';

import './xl-react-widget-tree-select.less';
import 'react-select/dist/react-select.css';

export class XlReactWidgetTreeSelect extends Component {
    static propTypes = {
        clearable: PropTypes.bool,
        data: PropTypes.arrayOf(PropTypes.shape({
            id: PropTypes.string.isRequired,
            title: PropTypes.string.isRequired
        })),
        expandAll: PropTypes.bool,
        indentationPadding: PropTypes.number,
        onModelChange: PropTypes.func,
        selectedFolderId: PropTypes.string
    };

    static defaultProps = {clearable: true, data: []};

    static getDerivedStateFromProps(props, state) {
        const {data, expandAll, selectedFolderId} = props;
        if (state.selectedFolderId === selectedFolderId) {
            return null;
        }
        return {
            ...state,
            selectedFolderId: selectedFolderId,
            tree: setSelectedNode(initTree(data, expandAll), selectedFolderId, expandAll)
        };
    }

    constructor(props) {
        super(props);
        this.dropdownRef = React.createRef();
        this.focusedItemRef = React.createRef();
        this.state = this.initialState();
    }

    componentDidUpdate() {
        this.scroll2item();
    }

    /* eslint-disable complexity */
    handleItemSelect = (folderSelected) => {
        const {expandAll, selectedFolderId} = this.props;
        const {tree} = this.state;

        if (!R.pathEq(['id'], selectedFolderId, folderSelected)) {
            const selectedId = R.prop('id', folderSelected);
            const selectedTitle = R.prop('title', folderSelected);
            const isSelectable = R.prop('enabled', folderSelected);

            this.setState({tree: setSelectedNode(tree, selectedFolderId, expandAll)});

            if (this.props.onModelChange && isSelectable) {
                this.props.onModelChange({selectedId, title: selectedTitle});
            } else if (R.isNil(selectedId)) {
                this.props.onModelChange({selectedId, title: ''});
            }
        }
    };
    /* eslint-enable */

    handleBlur = () => {
        const {expandAll, selectedFolderId} = this.props;
        if (selectedFolderId) {
            const {tree} = this.state;
            this.setState({tree: setSelectedNode(tree, selectedFolderId, expandAll)});
        }
    };

    handleItemToggle = (id, event) => {
        event.stopPropagation();
        const {tree} = this.state;
        this.setState({tree: toggleNode(tree, id)});
    };

    handleInputChange = (inputValue) => {
        const {data, expandAll, selectedFolderId} = this.props;

        if (inputValue.trim()) {
            this.setState({tree: filterTree(data, inputValue)});
        } else if (selectedFolderId) {
            this.setState({tree: setSelectedNode(initTree(data, expandAll), selectedFolderId, expandAll)});
        } else {
            this.setState({tree: initTree(data, expandAll)});
        }

        return inputValue;
    };

    /* eslint-disable complexity, default-case, max-depth */
    handleKeyDown = (event) => {
        const {selectedFolderId} = this.props;
        const {tree} = this.state;

        if (!this.dropdownRef.current._selectRef || !this.dropdownRef.current._selectRef._focusedOption) {
            return;
        }
        const {_focusedOption: {id: focusedId}} = this.dropdownRef.current._selectRef;

        const focused = focusedId || selectedFolderId || 0;
        const {keyCode} = event;
        const folders = flatTree(tree);

        const idx = folders.findIndex((folder) => folder.id === focused);

        if (idx >= 0) {
            switch (keyCode) {
                // [enter]
                case 13:
                    if (this.dropdownRef.current._selectRef.state.isOpen) {
                        event.preventDefault();
                        this.dropdownRef.current._selectRef.selectValue(folders[idx]);
                    }
                    break;
                // ←
                case 37: {
                    event.preventDefault();
                    const collapsed = toggleNode(tree, focused, false);
                    this.setState({tree: collapsed});
                    break;
                }
                // ↑
                case 38: {
                    event.preventDefault();
                    const nextItemIdx = idx === 0 ? folders.length - 1 : idx - 1;
                    this.dropdownRef.current._selectRef.focusOption(folders[nextItemIdx]);
                    this.setState({
                        needScrollTo: idx < nextItemIdx && nextItemIdx === folders.length - 1 ?
                            SCROLL_TO_BOTTOM :
                            undefined
                    });
                    break;
                }
                // →
                case 39: {
                    event.preventDefault();
                    const expanded = toggleNode(tree, focused, true);
                    this.setState({tree: expanded});
                    break;
                }
                // ↓
                case 40: {
                    event.preventDefault();
                    const nextItemIdx = idx === folders.length - 1 ? 0 : idx + 1;
                    this.dropdownRef.current._selectRef.focusOption(folders[nextItemIdx]);
                    this.setState({
                        needScrollTo: idx > nextItemIdx && nextItemIdx === 0 ?
                            SCROLL_TO_TOP :
                            undefined
                    });
                    break;
                }
            }
        }
    };
    /* eslint-enable */

    handleSelect = (selectValue, option) => () => {
        selectValue(option);
    };

    handleFocus = (focusOption, option) => () => {
        focusOption(option);
    };

    assignFocusedItem = (option, focusedOption) => (ref) => {
        if (option.id === focusedOption.id) {
            this.focusedItemRef = ref;
        }
    };

    /* eslint-disable react/no-find-dom-node */
    scroll2item = () => {
        const {needScrollTo} = this.state;
        const container = ReactDOM
            .findDOMNode(this.dropdownRef.current)
            .getElementsByClassName('VirtualSelectGrid')[0];
        changeScrollPosition(this.focusedItemRef, container, needScrollTo);
    };
    /* eslint-enable */

    initialState = () => {
        const {data, expandAll, selectedFolderId} = this.props;
        const tree = initTree(data, expandAll);
        return {
            selectedFolderId,
            tree: selectedFolderId ? setSelectedNode(tree, selectedFolderId, expandAll) : tree
        };
    };

    optionRenderer = (params) => {
        const {indentationPadding} = this.props;
        const {
            key, option, focusedOption, focusOption, selectValue, style
        } = params;

        return <XlReactWidgetTreeSelectItem
            indentationPadding={indentationPadding}
            isFocused={option.id === focusedOption.id}
            key={key}
            node={option}
            onMouseEnter={this.handleFocus(focusOption, option)}
            onSelect={this.handleSelect(selectValue, option)}
            onToggle={this.handleItemToggle}
            ref={this.assignFocusedItem(option, focusedOption)}
            style={style}
        />;
    };

    render() {
        const {selectedFolderId} = this.props;
        const {tree} = this.state;

        return <VirtualizedSelect
            clearable={this.props.clearable}
            filterOptions={(options) => options}
            labelKey="title"
            onBlur={this.handleBlur}
            onBlurResetsInput={false}
            onChange={this.handleItemSelect}
            onInputChange={this.handleInputChange}
            onInputKeyDown={this.handleKeyDown}
            optionHeight={29}
            optionRenderer={this.optionRenderer}
            options={flatTree(tree)}
            ref={this.dropdownRef}
            simpleValue={false}
            value={selectedFolderId}
            valueKey="id"
        />;
    }
}
